"""Base step class that all steps inherit from."""

# Import built-in modules
import abc
import json
import logging
import os

# Import third-party modules
import nuke  # pylint: disable=import-error

# Import local modules
from smartRescue.constants import LOGGING_LEVEL


class BaseStep(object):
    """Base class for our steps."""

    __metaclass__ = abc.ABCMeta

    def __init__(self, path, setup, job):
        """Initialize the BaseStep instance.

        Args:
            path (str): Absolute file path to operate on.
            setup (dict): Setup to use when running the step.
            job (dict): Configuration for the job to apply.

        """
        self.path = path
        self.setup = setup

        self.logger = logging.getLogger(self.name)
        self.logger.setLevel(LOGGING_LEVEL[job["logging"]])

    def __str__(self):
        """Return string representation of instance including setup to use.

        Returns:
            str: String representation of instance including setup to use.

        """
        return ("Step name: {}\n"
                "Setup: {}\n"
                "".format(self.name, repr(self.setup)))

    @property
    def parent_dir(self):
        """Get the parent directory of the member's path.

        We need to use a property in here and execute when needed. We cannot
        simply set the parent dir in the constructor, because the path member
        can change on runtime and wouldn't be re-evaluated if we initialized it
        in the constructor.

        Returns:
            str: Absolute path of the parent directory of the member's path.

        """
        return os.path.split(self.path)[0]

    @property
    def basename(self):
        """Get the basename of the member's path.

        We need to use a property in here and execute when needed. We cannot
        simply set the basename in the constructor, because the path member can
        change on runtime and wouldn't be re-evaluated if we initialized it in
        the constructor.

        Returns:
            str: The basename of the member's path.

        """
        filename = os.path.split(self.path)[1]
        return os.path.splitext(filename)[0]

    @property
    def name(self):
        """Return the name of this class.

        Returns:
            str: The name of this class.

        """
        return self.__class__.__name__

    def set_path(self, path):
        """Set path to given path.

        Args:
            path (str): Absolute path to use for process.

        """
        self.path = path

    def run(self, perform_file_handle=False):
        """Run the step.

        Args:
            perform_file_handle (bool, optional): If True perform additional
                functionality, otherwise pass it.

        """
        if perform_file_handle:
            pass

        self.process()

    @abc.abstractmethod
    def process(self):
        """Abstract process method that must be implemented.

        In here we describe all logic to perform for the step.

        """
        pass


class NodeStep(BaseStep):
    """Step that operates on nodes in the DAG by opening the Nuke script."""

    __metaclass__ = abc.ABCMeta

    def handle_node(self, node):
        """Disable or delete the given node, depending on set up.

        Raises:
            ValueError: When the mode is node supported.

        """
        mode = self.setup["mode"]

        if mode == "disable":
            disable_knob = node.knob("disable")
            if disable_knob:
                disable_knob.setValue(True)
        elif mode == "delete":
            nuke.delete(node)
        elif mode == "disconnect":
            node.selectOnly()
            nuke.extractSelected()
        else:
            raise ValueError("unsupported mode '{}'".format(mode))

    def run(self, perform_file_handle=False):
        """Run the step performing file handle.

        Args:
            perform_file_handle (bool, optional): If True perform open save and
                close operations on the Nuke script. We don't want this for our
                tests.

        """
        if perform_file_handle:
            self.logger.debug("Opening %s", self.path)
            nuke.scriptOpen(self.path)

        self.logger.debug("Using setup: %s", json.dumps(self.setup, indent=4))
        self.process()

        if perform_file_handle:
            self.logger.debug("Closing %s", self.path)
            nuke.scriptSave("")
            nuke.scriptClose()

    @abc.abstractmethod
    def process(self):
        """Abstract process method that must be implemented.

        In here we describe all logic to perform for the step.

        """
        pass
