"""Controller to drive the view."""

# Import built-in modules
import ast
import json
import logging
import os
import subprocess
import sys
import tempfile
import traceback

# Import local modules
from smartRescue import helper
from smartRescue import path_resolver
from smartRescue.constants import CRAGL_HOME_URL
from smartRescue.constants import DOCS_URL
from smartRescue.constants import LOGGING_LEVEL
from smartRescue.constants import STEPS
from smartRescue.constants import CRAGL_SMARTRESCUE_PROCESS_PATH
# Importing custom_widgets to have it in the sys modules so that we can access
# all the custom widget classes in StepWidgetController.build_setup_widgets.
from smartRescue.mvc import custom_widgets  # pylint: disable=unused-import
from smartRescue.mvc import dialogs
from smartRescue.mvc import model
from smartRescue.mvc import widgets
from smartRescue.mvc import gui_helper

# Icons for this package
ICONS = helper.load_icons()


class Controller(object):
    """Controller to drive the view."""

    def __init__(self, view, model_, standalone):
        """Initialize the Controller instance.

        Args:
            view (batch_process.mvc.view.View): The view to drive.
            model_ (batch_process.mvc.model.Model): The configuration instance.
            standalone (bool, optional): If True, run the main window in
                standalone mode, otherwise it is run via a DCC.

        """
        self.view = view
        self.config = model_
        self.standalone = standalone
        self.path_resolver = path_resolver.PathResolver(self.config.nuke_root)

        self.logger = logging.getLogger(__name__)

        self.create_signals()
        self.step_widget_controllers = []
        self.populate_step_widgets()
        self.setup_view()
        self.set_tooltips()
        self.latest_path = ""

        if os.environ.get(CRAGL_SMARTRESCUE_PROCESS_PATH):
            self.drop_path(os.environ[CRAGL_SMARTRESCUE_PROCESS_PATH])

        self.update_config_on_widget_change()
        self.setup_initial_nuke_root()

    def create_signals(self):
        """Create signals."""
        # We need anonymous functions on here, otherwise we would refer to
        # different function objects which is not what we want.
        # pylint: disable=unnecessary-lambda
        self.view.key_pressed.connect(lambda key: self.handle_key_pressed(key))
        self.view.push_settings.clicked.connect(lambda: self.toggle_settings())
        self.view.push_flush_log.clicked.connect(lambda: self.set_output(
            "", flush=True))
        self.view.settings.push_test_nuke_root.clicked.connect(
            lambda: self.test_nuke_root(show_success_message=True))
        self.view.push_process.clicked.connect(lambda: self.process())
        self.view.push_browse_process_path.clicked.connect(
            lambda: self.browse_process_path())
        self.view.label_startup.file_dropped.connect(self.drop_path)
        self.view.label_startup.clicked.connect(
            lambda: self.browse_process_path(startup_label_clicked=True))
        self.view.push_save_log.clicked.connect(lambda: self.save_log())
        self.view.settings.push_web_cragl.clicked.connect(
            lambda: helper.open_website(CRAGL_HOME_URL))
        self.view.settings.push_web_docs.clicked.connect(
            lambda: helper.open_website(DOCS_URL))

    def setup_initial_nuke_root(self):
        """Ask for Nuke installation root when it is not set.

        This is done only when running in standalone mode and not having set
        the Nuke installation root.

        """
        if self.standalone and not self.config.nuke_root:
            path = dialogs.show_dialog_set_nuke_root()
            self.view.settings.input_nuke_root.setText(path)
            if not self.test_nuke_root(show_success_message=True):
                self.setup_initial_nuke_root()

    def update_config_on_widget_change(self):
        """Update the configuration on a widget change in the settings."""
        settings = self.view.settings
        path = self.config.path

        settings.combo_logging.currentIndexChanged.connect(
            lambda: model.update_general(path, "logging",
                                         settings.combo_logging.currentText()))
        settings.input_ignore.textChanged.connect(
            lambda: model.update_general(path, "ignore_file_prefix",
                                         settings.input_ignore.text()))
        settings.check_overwrite.stateChanged.connect(
            lambda: model.update_general(path, "overwrite",
                                         settings.check_overwrite.isChecked()))

        settings.check_report.stateChanged.connect(
            lambda: model.update_general(path, "create_report",
                                         settings.check_report.isChecked()))
        settings.input_nuke_root.textChanged.connect(
            lambda: model.update_general(path, "nuke_root_dir",
                                         settings.input_nuke_root.text()))

    def process(self):
        """Process all working files using all enabled steps."""
        # Ensure that all enabled steps have valid setups.
        try:
            self.check_invalid_setups()
        except ValueError as error:
            dialogs.show_message_box(self.view, error.message)
            return

        # Get the process folder or process file from the process input.
        try:
            process_folder = self.get_process_path()
        except (OSError, ValueError) as error:
            dialogs.show_message_box(self.view, error.message)
            return

        # Get the setups to apply from the step widgets and write process file.
        enabled_steps = self.get_enabled_stepwidgets()
        process_file, job = self.write_job(enabled_steps, process_folder)

        # Ensure the Nuke executable is set and found.
        if not self.test_nuke_root():
            return

        command = self.assemble_command(process_file)
        self.logger.debug("Using command %s", command)

        process = subprocess.Popen(command,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT)

        # We need stderr in here because we need to use it indirectly
        # Otherwise, we couldn't populate the log in the main window.
        stdout, stderr = process.communicate()  # pylint:disable=unused-variable, line-too-long

        self.view.text_log.setText(stdout)
        self.update_config_file()

        if job["create_report"]:
            with open(job["logging_path"], "w") as file_:
                file_.write(stdout)

    def update_config_file(self):
        """Update the setup for all steps in the config file."""
        for step_ctrl in self.step_widget_controllers:
            step_config = step_ctrl.widget.text_setup.toPlainText()
            model.update_step(self.config.path, step_ctrl.class_name,
                              "setup", ast.literal_eval(step_config))


    def write_job(self, step_controller, process_path):
        """Write the job to apply to the process into the temp directory.

        We use the current state of all enabled StepWidgets in here and write
        it to a process.json file in the temp directory. We pass the absolute
        path of the job json file then to Nuke's terminal mode which will then
        parse the json file. We need to create the jobs in a delayed steps
        and cannot create them right here because they require the nuke module
        which is not available at this stage. The nuke module will only be
        available when launching Nuke in terminal mode.

        Args:
            widgets (smartRescue.mvc.controller.StepWidgetController): The
                StepWidget controller that we read data from.
            process_path (str): absolute path of folder to process.

        Returns:
            str, dict: Absolute path of process.json file that contains all
                setups to apply when processing in the format:
                    {
                        "process_path": "path/to/folder/to/process",
                        "logging": "debug",
                        "ignore_file_prefix": "_",
                        "create_report": true,
                        "overwrite": false
                        "steps": [
                            [
                                {
                                    "_class_name": "AllNodes",
                                    "mode": "disable",
                                    "keep": []
                                },
                                ...
                            ]
                        ]
                    }

        """
        temp_path = os.path.join(tempfile.gettempdir(), "cragl_smartrescue")
        date_now = helper.date_now()

        dir_ = os.path.dirname(process_path) if os.path.isfile(process_path) \
            else process_path

        job_path = os.path.join(temp_path, date_now)
        setups_path = os.path.join(job_path, "job.json")
        logging_path = os.path.join(dir_, "{}_job.log".format(date_now))

        job = {
            "logging": self.view.settings.combo_logging.currentText(),
            "logging_path": logging_path,
            "ignore_file_prefix": self.view.settings.input_ignore.text(),
            "overwrite": self.view.settings.check_overwrite.isChecked(),
            "create_report": self.view.settings.check_report.isChecked(),
            "process_path": process_path,
            "job_configuration_path": setups_path
        }

        try:
            os.makedirs(job_path)
        except OSError:
            pass

        setups = []
        for ctrl in step_controller:
            data = ctrl.get_setup()
            data["_class_name"] = ctrl.class_name
            setups.append(data)

        job["steps"] = setups

        with open(setups_path, "w") as file_:
            json.dump(job, file_)

        return setups_path, job

    def assemble_command(self, process_file):
        """Assemble the command to run in a subprocess.

        Returns:
            list: Command arguments to run in a subprocess.

        """
        this_dir = os.path.dirname(__file__)
        process_mod = os.path.join(this_dir, "..", "process.py")
        process_mod = os.path.abspath(process_mod)

        if self.standalone:
            executable = self.path_resolver.nuke
        else:
            import nuke  # pylint: disable=import-error
            executable = nuke.env["ExecutablePath"]

        return [
            executable,
            "-i",
            "-t",
            process_mod,
            process_file
        ]

    def save_log(self):
        """Show the file browser to save the current log."""

        title = "Save log"
        path = dialogs.save_to_disk(title)

        if path:
            # Ensure we save a ".txt" file.
            path = helper.ensure_file_extension(path, "txt")
            content = self.view.text_log.toPlainText()

            with open(path, "w") as file_:
                file_.write(content)

    def drop_path(self, path):
        """Handle the drop of the given path.

        Manage visibility of groups and set up the process path.

        """
        if os.path.isfile(path):
            if not path.endswith(".nk"):
                message = "Invalid file extension. Only .nk files are allowed."
                dialogs.show_message_box(self.view, message)
                return

        self.view.group_startup.setVisible(False)
        self.view.group_main.setVisible(True)

        self.view.input_process_path.setText(path)

    def get_process_path(self):
        """Get the folder or file to process.

        Returns:
            str: Absolute path of folder or file to process.

        Raises:
            ValueError: When no path is set or the path is no .nk file.
            OSError: When the process folder does not exist.

        """
        path = self.view.input_process_path.text()

        if not path:
            raise ValueError("Please set a process folder path.")

        if os.path.isfile(path):
            if os.path.splitext(path)[1] == ".nk":
                return path
            raise ValueError("Invalid file extension. Please select a .nk "
                             "file.")

        if not os.path.isdir(path):
            raise OSError("The selected folder to process does not exist.")

        return path

    def get_enabled_stepwidgets(self):
        """Get all enabled steps.

        Returns:
            list: All enabled steps.

        """
        return [ctrl for ctrl in self.step_widget_controllers
                if ctrl.is_checked()]

    def check_invalid_setups(self):
        """"Check for invalid setups prior to executing.

        Raises:
            ValueError: When any setup of an enabled step is invalid.

        """
        invalid_setups = self._get_invalid_setups()
        if invalid_setups:
            message = ("The following setups are invalid: \n\n {} \n\n"
                       "Please fix these before proceeding.")
            raise ValueError(message.format("\n".join(invalid_setups)))

    def _get_invalid_setups(self):
        """Get all controllers that have invalid setups.

        Returns:
            list: Names of steps with invalid setups.

        """
        invalid_setup = []

        for ctrl in self.step_widget_controllers:
            ctrl.validate_setup(show_traceback=False)

            if not ctrl.setup_is_valid:
                invalid_setup.append(ctrl.class_name)

        return invalid_setup

    def browse_process_path(self, startup_label_clicked=False):
        """Show browser to browse to the path to process.

        Args:
            startup_label_clicked (bool, optional): When True the user emited
                this function from clicking the startup label. In that case
                perform extra logic. If False, return without executing custom
                functionality for the startup label.

        """
        path = dialogs.show_path_browser("Set process directory")
        if path:
            self.view.input_process_path.setText(path)
            if startup_label_clicked:
                self.drop_path(path)

    def set_tooltips(self):
        """Set tooltips for all major widgets."""
        view = self.view
        gui_helper.set_tooltip(view.push_settings, "settings")
        gui_helper.set_tooltip(view.push_save_log, "save")
        gui_helper.set_tooltip(view.push_flush_log, "erase")
        gui_helper.set_tooltip(view.push_process, "process")

        settings = self.view.settings
        gui_helper.set_tooltip(settings.label_logging, "logging")
        gui_helper.set_tooltip(settings.combo_logging, "logging")
        gui_helper.set_tooltip(settings.label_ignore, "ignore")
        gui_helper.set_tooltip(settings.input_ignore, "ignore")
        gui_helper.set_tooltip(settings.check_overwrite, "overwrite")
        gui_helper.set_tooltip(settings.check_report, "report")
        gui_helper.set_tooltip(settings.label_nuke_root, "nuke_root")
        gui_helper.set_tooltip(settings.input_nuke_root, "nuke_root")
        gui_helper.set_tooltip(settings.push_test_nuke_root, "test_nuke_root")

    def test_nuke_root(self, show_success_message=False):
        """Test if we can find the nuke executable.

        If we can't find any Nuke executable, we show a pop up window informing
        the user about it.

        Args:
            show_success_message (bool, optional): If True show a success
                message when the Nuke executable was found. This is useful
                when pressing the test button, but we don't want a success
                message when processing.

        Returns:
            bool: True when a Nuke executable has been found.

        """
        # When running in Nuke, then the Nuke executable is always available.
        if not self.standalone:
            return True

        path = self.view.settings.input_nuke_root.text()
        self.path_resolver.set_path(path)

        try:
            self.path_resolver.run()  # Might potentially raise here.
            message = ("Successfully found Nuke executable in:\n"
                       "{}".format(self.path_resolver.nuke))
            if show_success_message:
                dialogs.show_message_box(self.view, message)
            return True

        except OSError as error:
            message = error.message
            dialogs.show_message_box(self.view, message)

    def setup_view(self):
        """Setup all view widgets."""
        settings = self.view.settings

        logging_level = [level for level in LOGGING_LEVEL]
        settings.combo_logging.addItems(logging_level)

        if self.config.verbosity in logging_level:
            current_logging_level = self.config.verbosity
        else:
            current_logging_level = "info"
        settings.combo_logging.setCurrentIndex(
            settings.combo_logging.findText(current_logging_level))

        settings.input_ignore.setText(self.config.ignore_prefix)
        settings.check_overwrite.setChecked(self.config.overwrite)
        settings.check_report.setChecked(self.config.report)
        settings.input_nuke_root.setText(self.config.nuke_root)

        environ_process_path = os.environ.get(CRAGL_SMARTRESCUE_PROCESS_PATH)
        if environ_process_path:
            self.view.input_process_path.setText(environ_process_path)

        widgets_ = (
            self.view.settings.label_nuke_root,
            self.view.settings.input_nuke_root,
            self.view.settings.push_test_nuke_root
        )
        for widget in widgets_:
            widget.setVisible(self.standalone)

    def toggle_settings(self):
        """Toggle show/ hide the settings section."""
        self.view.group_settings.setVisible(
            not self.view.group_settings.isVisible())

        self.view.group_steps_wrapper.setVisible(
            True)

    def handle_key_pressed(self, key):
        """Handle key presses events.

        Args:
            key (str): Name of the key being pressed.

        """
        actions = {
            "esc": self.view.close
        }
        actions[key]()

    def populate_step_widgets(self):
        """Populate view with step widgets and connect to controllers."""
        for step in self.config.data[STEPS]:
            step_widget = widgets.StepWidget(self)

            ctrl = StepWidgetController(
                step_widget,
                step["class"],
                step["enable"],
                step["description"],
                step["setup"],
                step["widgets"],
                self
            )
            self.view.layout_steps_stepwidgets.addWidget(step_widget)
            self.step_widget_controllers.append(ctrl)

    def set_output(self, text, flush=False):
        """Set given text in log output.

        Args;
            text (str): Text to set in the log output text area.
            flush (bool, optional): If True flush the the log output text area,
                otherwise append to it.

        """
        if flush:
            self.view.text_log.setText("")
        self.view.text_log.setText(self.view.text_log.toPlainText() + text)


class StepWidgetController(object):
    """Controller for a StepWidget instance."""

    def __init__(self, widget, class_name, is_enabled, description, setup,
                 setup_widgets, master_controller):
        """Initialize the StepWidgetController instance.

        Args:
            widget (smartRescue.mvc.widgets.StepWidget): The StepWidget
                instance to drive.
            class_name (str): Class name of the step. We will do a lookup when
                creating the actual underlying step.
            is_enabled (bool): If True set the checkbox checked, otherwise set
                it unchecked.
            description (str): Description of the step to use as a tooltip.
            setup (dict): Setup to show in the setup text area.
            setup_widgets (list of dict): Widgets to create for the stepWidget.
                has the format:
                [
                    {"name-of-widget": "widget_type:value"},
                    {"name-of-widget": "widget_type:value"},
                    ...
                ]
            master_controller (smartRescue.mvc.controller.Controller): The
                master controller that drives the main window. Needed to get
                the config path.

        """
        self.widget = widget
        self.class_name = class_name
        self.is_enabled = is_enabled
        self.description = description
        self.widgets_setup = setup_widgets
        self.master_controller = master_controller
        self.setup_is_valid = False
        self.mode = ""
        self.setup_widgets = {}

        self.setup = setup

        self.setup_step_widget()
        self.create_signals()

        # Auto resize to show the whole setup.
        self.update_text_setup_widget()

        self.build_setup_widgets()

    def __str__(self):
        """Return string representation for the controller.

        Returns:
            str: Representation for the controller.

        """
        data = (
            "widget: {}"
            "class_name: {}"
            "is_enabled: {}"
            "description: {}"
            "setup_widgets: {}"
            "master_controller: {}"
        )
        return data.format(self.widget, self.class_name, self.is_enabled,
                           self.description, self.widgets_setup,
                           self.master_controller)

    def create_signals(self):
        """Create signals."""
        # We need anonymous functions on here, otherwise we would refer to
        # different function objects which is not what we want.
        # pylint: disable=unnecessary-lambda
        self.widget.push_toggle_body.clicked.connect(
            lambda: self.toggle_body())
        self.widget.push_test.clicked.connect(lambda: self.validate_setup())
        self.widget.text_setup.textChanged.connect(
            lambda: self.update_text_setup_widget())
        self.widget.check.stateChanged.connect(
            lambda: self.update_enabled_in_config())
        self.widget.push_info.clicked.connect(self.toggle_info)
        self.widget.push_toggle_mode.clicked.connect(
            lambda: self.toggle_mode())

    def build_setup_widgets(self):
        """Build the widgets for the StepWidget from step_widgets config.

        Raises:
            KeyError: When any widget class name from the setup widgets does
                not exist.

        """
        if self.widgets_setup:
            self.widget.text_setup.setVisible(False)
            self.mode = "standard"
            icon = ICONS["mode_standard"]
            for widget_setup in self.widgets_setup:
                type_ = widget_setup["type"]
                try:
                    class_ = getattr(
                        sys.modules["smartRescue.mvc.custom_widgets"], type_)
                except AttributeError:
                    error = ("No such widget class type '{}' found "
                             "in smartRescue.mvc.custom_widgets".format(type_))
                    raise AttributeError(error)

                widget = class_(widget_setup, self.setup)

                self.add_widget(widget)
                self.setup_widgets[widget_setup["id"]] = widget
        else:
            self.mode = "advanced"
            icon = ICONS["mode_advanced"]
            self.widget.group_setup_widgets.setVisible(False)

        gui_helper.set_icon(self.widget.push_toggle_mode, icon)

    def add_widget(self, widget):
        """Add the given widget to the widget's layout_setup_widgets.

        Args:
            widget (smartRescue.mvc.custom_widgets.<Widget>): Custom widget to
                add.

        """
        self.widget.layout_setup_widgets.addWidget(widget)

    def get_setup(self):
        """Get the setup of the underlying widget.

        When being in advanced mode, then we parse the text_setup widget' text,
        otherwise we read out the values of all the custom widgets and create
        the setup from these widgets values.

        Returns:
            dict: The setup of the underlying widget.

        """
        # Get the setup from all custom widgets.
        if self.mode == "standard":
            setup = {}
            for key, widget in self.setup_widgets.items():
                setup[key] = widget.value()
            return setup

        # Get the setup from the text setup widget.
        elif self.mode == "advanced":
            content = self.widget.text_setup.toPlainText()
            if not content:
                return {}
            return ast.literal_eval(content)

    def is_checked(self):
        """Return checked status of view's checkbox.

        Returns:
            bool: True when the view's check is checked, otherwise False.

        """
        return self.widget.check.isChecked()

    def set_info_from_docstring(self):
        """Set info from doc string if '$DOC' is in the description."""
        if "$DOC:" in self.description:
            name = self.description.split(":")[1]
            this_dir = os.path.dirname(__file__)
            path = os.path.join(this_dir, "..", "steps", "{}.py".format(name))
            if not os.path.isfile(path):
                raise IOError("No such module to parse doc string for: "
                              "{}".format(path))
            docstring_elements = helper.get_docstring_elements(path)

            docs_header = docstring_elements[0]
            docs_body = docstring_elements[1]
            docs_example_standard = docstring_elements[2]
            docs_example_advanced = docstring_elements[3]

            self.widget.label_docs_header.setText(docs_header)
            self.widget.label_docs_body.setText(docs_body)
            self.widget.label_docs_example_standard.setText(
                docs_example_standard)
            self.widget.label_docs_example_advanced.setText(
                docs_example_advanced)

        else:
            self.widget.label_docs_header.setText(self.description)
            self.widget.push_info.setVisible(False)

    def setup_step_widget(self):
        """Set up the widget."""
        self.widget.label_docs_header.setText(self.class_name)
        self.widget.check.setText(self.class_name)
        self.widget.check.setChecked(self.is_enabled)
        self.widget.text_setup.setText(json.dumps(self.setup, indent=4))
        self.set_info_from_docstring()

    def toggle_mode(self):
        """Toggle mode between standard and advanced."""
        # Switch to 'advanced' mode.
        if self.mode == "standard":
            self.mode = "advanced"
            icon = ICONS["mode_advanced"]

            self.widget.text_setup.setVisible(True)
            self.widget.group_setup_widgets.setVisible(False)
            self.widget.push_test.setVisible(True)

        # Switch to 'standard' mode.
        else:
            self.mode = "standard"
            icon = ICONS["mode_standard"]

            self.widget.group_setup_widgets.setVisible(True)
            self.widget.text_setup.setVisible(False)
            self.widget.push_test.setVisible(False)

        gui_helper.set_icon(self.widget.push_toggle_mode, icon)

        self.show_example(self.mode)

    def show_example(self, which):
        """Show the given example setup

        Args:
            which (str): Name of the example to show ("standard", "advanced").

        """
        self.widget.label_docs_example_standard.setVisible(which == "standard")
        self.widget.label_docs_example_advanced.setVisible(which != "standard")

    def toggle_info(self):
        """Toggle the information detail."""
        self.widget.label_docs_body.setVisible(
            not self.widget.label_docs_body.isVisible())

        self.widget.group_docs_examples.setVisible(
            self.widget.label_docs_body.isVisible())

        if self.widget.label_docs_body.isVisible():
            self.show_example(self.mode)

    def update_enabled_in_config(self):
        """Update the 'enable' state in the config for the current step."""
        model.update_step(self.master_controller.config.path, self.class_name,
                          "enable", self.is_checked())

    def update_text_setup_widget(self):
        """Perform validation and resize the text_setup text area."""
        self.validate_setup(show_traceback=False)
        gui_helper.resize_textedit(self.widget.text_setup)

    def validate_setup(self, show_traceback=True):
        """Check if the setup is well formed and update icon.

        Args:
            show_traceback (bool, optional): If True show the traceback in the
                log output.

        """
        try:
            ast.literal_eval(self.widget.text_setup.toPlainText())
            icon = ICONS["test_green"]
            self.widget.text_setup.setProperty("style", "text_setup_green")
            self.setup_is_valid = True
        except (SyntaxError, ValueError):
            icon = ICONS["test_red"]
            self.widget.text_setup.setProperty("style", "text_setup_red")
            self.setup_is_valid = False

            if show_traceback:
                self.widget.parent_controller.set_output(
                    traceback.format_exc(), flush=False)

        gui_helper.set_icon(self.widget.push_test, icon)
        gui_helper.set_style_sheet(self.widget.text_setup)

    def toggle_body(self):
        """Toggle show/ hide the widget's body and update icon."""
        self.widget.group_body.setVisible(
            not self.widget.group_body.isVisible())

        if self.widget.group_body.isVisible():
            icon = ICONS["arrow_up"]
        else:
            icon = ICONS["arrow_down"]
        gui_helper.set_icon(self.widget.push_toggle_body, icon)

        self.widget.push_test.setVisible(self.mode == "advanced")
