"""Custom widgets for our StepWidgets.

These classes can be used for widgets in the StepWidget' standard mode. These
are confifgured in the config.json file for each step in the 'widgets' section.

Each class in here must implement a 'value' method to get the current value
from the widget.

"""

# Import built-in modules
import uuid

# Import third-party modules
try:
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtWidgets

# Import local modules
from smartRescue.constants import COMBO_CHOICES
from smartRescue.mvc import gui_helper
from smartRescue import helper

# Icons for this package.
ICONS = helper.load_icons()


class IntegerInput(QtWidgets.QWidget):
    """Custom integer input field with label."""

    def __init__(self, widget_setup, setup):
        """Initialize the IntegerInput instance.

        Args:
            widget_setup (dict): Setup to apply to the widget in the format:
                {
                    "id": "<id-to-set-as-object-name>",
                    "label": "label-to-apply",
                    "tooltip": "tooltip-to-apply",
                }
            setup (dict): Setup to grab default values from in the example
                format:
                {
                    "first_line": 1,
                    "last_line": 64
                }

        """
        super(IntegerInput, self).__init__()

        self.widget_setup = widget_setup
        self.setup = setup

        self.setObjectName(self.widget_setup["id"])

        self.label = QtWidgets.QLabel(self.widget_setup["label"])
        self.label.setToolTip(self.widget_setup["tooltip"])

        self.input = QtWidgets.QLineEdit()
        self.input.setToolTip(widget_setup["tooltip"])
        placeholder = self.widget_setup.get("placeholder")
        if not placeholder:
            placeholder = [""]
        self.input.setPlaceholderText(placeholder[0])

        startup_value = setup.get(widget_setup["id"], 1)
        self.input.setText(str(startup_value))

        self.layout = QtWidgets.QHBoxLayout()
        self.layout.addWidget(self.label)
        self.layout.addWidget(self.input)

        self.setLayout(self.layout)

    def value(self):
        """Get the value from the input.

        Returns:
            int: The current value from the input.

        """
        return int(self.input.text()) or 1


class Dropdown(QtWidgets.QWidget):
    """Custom dropdown menu with label.

    If a list for widget_setup['values'] exists, it will be set to the combo's
    choices.

    """

    def __init__(self, widget_setup, setup):
        """Initialize the Dropdown instance.

        Args:
            widget_setup (dict): Setup to apply to the widget in the format:
                {
                    "id": "<id-to-set-as-object-name>",
                    "label": "label-to-apply",
                    "tooltip": "tooltip-to-apply",
                }
            setup (dict): Setup to grab default values from in the example
                format:
                {
                    "skip": [],
                    "mode": "delete"
                }

        """
        super(Dropdown, self).__init__()

        self.widget_setup = widget_setup
        self.setup = setup

        self.setObjectName(self.widget_setup["id"])

        self.label = QtWidgets.QLabel(self.widget_setup["label"])
        self.label.setToolTip(self.widget_setup["tooltip"])

        self.combo = QtWidgets.QComboBox()
        self.combo.setToolTip(widget_setup["tooltip"])

        self.combo.addItems(widget_setup.get("values", ""))

        startup_value = setup.get(widget_setup["id"], "")
        index = self.combo.findText(startup_value)
        self.combo.setCurrentIndex(index)

        self.layout = QtWidgets.QHBoxLayout()
        self.layout.addWidget(self.label, 1)
        self.layout.addWidget(self.combo, 9)

        self.setLayout(self.layout)

    def value(self):
        """Get the current value from the combo.

        Returns:
            str: Current value from the combo.

        """
        return self.combo.currentText()


class HandleDropdown(Dropdown):
    """Custom dropdown menu setup with options to handle nodes.

    The choices to offer in the drop down menu are loaded from
    constants.COMBO_CHOICES.

    """

    def __init__(self, widget_setup, setup):
        """Initialize the HandleDropdown instance.

        Args:
            widget_setup (dict): Setup to apply to the widget in the format:
                {
                    "id": "<id-to-set-as-object-name>",
                    "label": "label-to-apply",
                    "tooltip": "tooltip-to-apply",
                }
            setup (dict): Setup to grab default values from in the example
                format:
                {
                    "skip": [],
                    "mode": "delete"
                }

        """
        super(HandleDropdown, self).__init__(widget_setup, setup)
        self.combo.addItems(COMBO_CHOICES)

        mode = self.setup.get("mode")
        if mode:
            self.combo.setCurrentIndex(self.combo.findText(mode))


class _RowSingle(QtWidgets.QWidget):
    """One row that includes an input and a minus button to remove it."""

    def __init__(self, value, placeholder):
        """Initialize the _RowSingle instance.

        Args:
            value (str): Value to apply to the input.
            placeholder (str): Placeholder text to use in the input.

        """
        super(_RowSingle, self).__init__()

        self.id_ = str(uuid.uuid4())
        self.setObjectName(self.id_)

        self.layout_row = QtWidgets.QHBoxLayout()

        self.input = QtWidgets.QLineEdit(value)
        self.input.setPlaceholderText(placeholder)
        self.minus = QtWidgets.QToolButton()
        self.minus.setObjectName(self.id_)
        gui_helper.set_icon(self.minus, ICONS["minus"])

        self.layout_row.addWidget(self.input, 9)
        self.layout_row.addWidget(self.minus, 1)

        self.setLayout(self.layout_row)

    def value(self):
        """Return the current value of the input.

        Returns:
            str: The current value of the input.

        """
        return self.input.text().strip()


class _RowDouble(QtWidgets.QWidget):
    """One row that includes two inputs and a minus button to remove it."""

    def __init__(self, values, placeholder):
        """Initialize the _RowDouble instance.

        Args:
            value (str): Value to apply to the input.
            placeholder (str, str): Placeholder texts to apply to the inputs.

        """
        super(_RowDouble, self).__init__()

        self.id_ = str(uuid.uuid4())
        self.setObjectName(self.id_)

        self.layout_row = QtWidgets.QHBoxLayout()

        self.input1 = QtWidgets.QLineEdit(values[0])
        self.input1.setPlaceholderText(placeholder[0])
        self.input2 = QtWidgets.QLineEdit(values[1])
        self.input2.setPlaceholderText(placeholder[1])
        self.minus = QtWidgets.QToolButton()
        self.minus.setObjectName(self.id_)
        gui_helper.set_icon(self.minus, ICONS["minus"])

        self.layout_row.addWidget(self.input1, 9)
        self.layout_row.addWidget(self.input2, 9)
        self.layout_row.addWidget(self.minus, 1)

        self.setLayout(self.layout_row)

    def value(self):
        """Return the current value of the input.

        Returns:
            list: The current value of the input1 and input2.

        """
        return [self.input1.text().strip(), self.input2.text().strip()]


class _RowQuad(QtWidgets.QWidget):
    """One row that includes 4 inputs and a minus button to remove it."""

    def __init__(self, values, placeholder):
        """Initialize the _RowQuad instance.

        Args:
            value (dict, optional): Values to apply to the inputs.
            placeholder (str, str, str, str): Placeholder texts to apply to the
                inputs.

        """
        super(_RowQuad, self).__init__()

        self.id_ = str(uuid.uuid4())
        self.setObjectName(self.id_)

        self.layout_row = QtWidgets.QHBoxLayout()

        values = values or {
            "node_class": "",
            "knob_name": "",
            "operator": "",
            "knob_value": ""
        }

        self.input1 = QtWidgets.QLineEdit(values["node_class"])
        self.input2 = QtWidgets.QLineEdit(values["knob_name"])
        self.input3 = QtWidgets.QLineEdit(values["operator"])
        self.input4 = QtWidgets.QLineEdit(values["knob_value"])

        self.input1.setPlaceholderText(placeholder[0])
        self.input2.setPlaceholderText(placeholder[1])
        self.input3.setPlaceholderText(placeholder[2])
        self.input4.setPlaceholderText(placeholder[3])

        self.minus = QtWidgets.QToolButton()
        self.minus.setObjectName(self.id_)
        gui_helper.set_icon(self.minus, ICONS["minus"])

        self.layout_row.addWidget(self.input1, 9)
        self.layout_row.addWidget(self.input2, 9)
        self.layout_row.addWidget(self.input3, 9)
        self.layout_row.addWidget(self.input4, 9)
        self.layout_row.addWidget(self.minus, 1)

        self.setLayout(self.layout_row)

    def value(self):
        """Return the current value of the input.

        Returns:
            dict: The current values of the input1, input2, input3 and input4.

        """
        return {
            "node_class": self.input1.text().strip(),
            "knob_name": self.input2.text().strip(),
            "operator": self.input3.text().strip(),
            "knob_value": self.input4.text().strip()
        }


class InputArray(QtWidgets.QWidget):
    """Array of inputs to dynamically add/ remove values.

    This widget contains a plus button to add new rows. Each row contains a
    minus button to remove the currently clicked row.

    """

    def __init__(self, widget_setup, setup):
        """Initialize the InputArray Widget.

        Args:
            widget_setup (dict): Setup to apply to the widget in the format:
                {
                    "id": "<id-to-set-as-object-name>",
                    "label": "label-to-apply",
                    "tooltip": "tooltip-to-apply",
                }
            setup (dict): Setup to grab default values from in the example
                format:
                {
                    "skip": [],
                    "mode": "delete"
                }

        """
        super(InputArray, self).__init__()

        self.widget_setup = widget_setup
        self.setup = setup
        self.placeholder = self.widget_setup.get("placeholder", [""])

        self.setObjectName(self.widget_setup["id"])
        self.widgets = []

        self.create_widgets()
        self.create_layouts()
        self.setup_inputs()
        self.create_signals()

    def create_widgets(self):
        """Create widgets."""
        self.label = QtWidgets.QLabel(self.widget_setup["label"])
        self.label.setToolTip(self.widget_setup["tooltip"])

        self.plus = QtWidgets.QToolButton()
        gui_helper.set_icon(self.plus, ICONS["plus"])

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_label = QtWidgets.QVBoxLayout()
        self.layout_label.addWidget(self.label)
        self.layout_label.addStretch()

        self.layout_value_inputs = QtWidgets.QVBoxLayout()

        self.layout_plus = QtWidgets.QHBoxLayout()
        self.layout_plus.addStretch()
        self.layout_plus.addWidget(self.plus)

        self.layout_rows = QtWidgets.QHBoxLayout()
        self.layout_rows.addLayout(self.layout_label)
        self.layout_rows.addLayout(self.layout_value_inputs)

        self.layout_main.addLayout(self.layout_rows)
        self.layout_main.addLayout(self.layout_plus)

        self.setLayout(self.layout_main)
        gui_helper.set_style_sheet(self)

    def create_signals(self):
        """Create signals."""
        # We need anonymous functions on here, otherwise we would refer to
        # different function objects which is not what we want.
        # pylint: disable=unnecessary-lambda
        self.plus.clicked.connect(lambda: self.add())

    def setup_inputs(self):
        """Create an input for each start value."""

        for value in self.setup[self.widget_setup["id"]]:
            self.add(value)

    def add(self, value=""):
        """Add additional input.

        Args:
            value (str): Value to apply to the input.

        """
        placeholder = self.widget_setup.get("placeholder")
        placeholder = placeholder[0] if placeholder else ""

        row = _RowSingle(value, placeholder)

        self.layout_value_inputs.addWidget(row)
        self.widgets.append(row)

        row.minus.clicked.connect(self.remove)

    def remove(self):
        """Remove the currently clicked row."""
        for widget in self.widgets:
            if widget.objectName() == self.sender().objectName():
                index = self.widgets.index(widget)
                widget.setVisible(False)
                del self.widgets[index]

    def value(self):
        """Sanitize and get values of all inputs.

        Returns:
            list: Sanitized values from all rows.

        """
        return [row.value() for row in self.widgets
                if row.value()]


class InputArrayDouble(InputArray):
    """Array of _RowDouble inputs to dynamically add/ remove values.

    This widget contains a plus button to add new rows. Each row contains a
    minus button to remove the currently clicked row.

    """

    def add(self, values=("", "")):
        """Add additional input using a _RowDouble widget.

        Args:
            value (list): Values to apply to the input1 and input2.

        """
        placeholder = self.widget_setup.get("placeholder")
        if not placeholder:
            placeholder = ["", ""]
        row = _RowDouble(values, placeholder)
        self.layout_value_inputs.addWidget(row)
        self.widgets.append(row)

        row.minus.clicked.connect(self.remove)


class InputArrayQuad(InputArray):
    """Array of _RowQuad inputs to dynamically add/ remove values.

    This widget contains a plus button to add new rows. Each row contains a
    minus button to remove the currently clicked row.

    """

    def add(self, values=None):
        """Add additional input using a _RowDouble widget.

        Args:
            value (dict, optional): Values to apply to the inputs.

        """
        placeholder = self.widget_setup.get("placeholder")
        if not placeholder:
            placeholder = ["", "", "", ""]

        row = _RowQuad(values, placeholder)
        self.layout_value_inputs.addWidget(row)
        self.widgets.append(row)

        row.minus.clicked.connect(self.remove)
