"""This module contains dialog windows classes for multiple occasions."""

# Import third-party modules
try:
    from PySide import QtGui as QtWidgets
    from PySide import QtCore
except ImportError:
    from PySide2 import QtWidgets
    from PySide2 import QtCore

# Import local modules
from smartRescue import helper
from smartRescue.mvc import gui_helper
from smartRescue.mvc.tooltips import TOOLTIPS

# Icons for this package.
ICONS = helper.load_icons()


def show_message_box(window, message):
    """Show message box with message as content.

    Args:
        window (QtWidgets.QWidget): Parent window in order to keep the message
            box on top of the parent window.
        message (str): Message to display in the window.

    """
    dialog = QtWidgets.QMessageBox()
    dialog.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
    dialog.information(window, 'information', message)


def show_path_browser(title):
    """Show browser to browse to path and return browsed path.

    Args:
        title (str): Title to display for browser.

    Returns:
        str: Absolute path that was browsed to.

    """
    dialog = QtWidgets.QFileDialog()
    dialog.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
    dialog.setWindowTitle(title)
    dialog.setFileMode(QtWidgets.QFileDialog.Directory)
    dialog.setOption(QtWidgets.QFileDialog.ShowDirsOnly)

    if dialog.exec_() == QtWidgets.QDialog.Accepted:
        return dialog.selectedFiles()[0]
    return ""


def save_to_disk(title, filter_="txt file(*.txt)"):
    """Create a save to file browser and save the file contents.

    Args:
        title (str): Title to display in the window.
        ext_filter (str): Extension to filter. Include a dot.

    """
    dialog = QtWidgets.QFileDialog()
    dialog.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
    dialog.setWindowTitle(title)
    dialog.setNameFilter(filter_)
    dialog.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)

    gui_helper.set_style_sheet(dialog)

    if dialog.exec_() == QtWidgets.QDialog.Accepted:
        return dialog.selectedFiles()[0]

# We want to keep the class more readable and not initialize all ui members in
# init method.
# pylint: disable=attribute-defined-outside-init
class DialogSetNukeRoot(QtWidgets.QDialog):
    """Dialog to help the user setting a Nuke root.

    This window shows up when no Nuke root has been set and when running in
    standalone mode. When running in Nuke, we will use the executable of the
    currently used Nuke version.

    """

    def __init__(self):
        """Initialize the DialogSetNukeRoot instance."""
        super(DialogSetNukeRoot, self).__init__()

        self.setup()
        self.build_ui()

        self.input_path.setFocus()

    def setup(self):
        """Set up dialog."""
        self.setWindowTitle("Enter Nuke installation root")
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        self.setFixedWidth(500)
        self.setModal(True)

    def build_ui(self):
        """Build ui."""
        self.create_widgets()
        self.create_layouts()
        self.create_signals()

    def create_widgets(self):
        """Create widgets."""
        self.input_path = QtWidgets.QLineEdit()
        self.push_browse = QtWidgets.QToolButton()
        self.label_info = QtWidgets.QLabel(TOOLTIPS["nuke_root"])
        self.label_info.setWordWrap(True)
        gui_helper.set_icon(self.push_browse, ICONS["folder"])
        self.push_ok = QtWidgets.QPushButton("OK")
        self.push_ok.setProperty("style", "action_button")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_browse = QtWidgets.QHBoxLayout()

        self.layout_browse.addWidget(self.input_path)
        self.layout_browse.addWidget(self.push_browse)

        self.layout_main.addLayout(self.layout_browse)
        self.layout_main.addWidget(self.label_info)
        self.layout_main.addWidget(self.push_ok)

        self.setLayout(self.layout_main)
        gui_helper.set_style_sheet(self)

    def create_signals(self):
        """Create signals."""
        self.push_browse.clicked.connect(self.show_browser)
        self.push_ok.clicked.connect(self.set_nuke_root)

    def show_browser(self):
        """Show browser to browse to Nuke installation root."""
        path = show_path_browser("Browse to Nuke installation root directory.")
        self.input_path.setText(path)

    def set_nuke_root(self):
        """Set Nuke root to the given currently set up path."""
        if self.input_path.text():
            self.accept()


def show_dialog_set_nuke_root():
    """Show dialog set the Nuke root.

    Returns:
        str: Md5 sum of entered password.

    """
    dialog = DialogSetNukeRoot()
    dialog.raise_()
    dialog.show()
    if dialog.exec_():
        return dialog.input_path.text()
