"""Gui related helper functionality."""

# Import built-in modules
import os

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui

# Import local modules
from smartRescue.mvc.tooltips import TOOLTIPS


def set_icon(widget, path):
    """Update the icon of the given widget to the given path.

    Args:
        widget (QtWidgets.QWidget): Widget to update the icon on.
        path (str): Absolute path of icon to update to.

    """
    widget.setIcon(QtGui.QIcon(path))


def set_style_sheet(widget, style="styles.qss"):
    """Apply css style sheet to given widget.

    Args:
        widget (QtWidgets.QWidget): Widget to apply styles to.
        style (str): Name of styles file to apply.

    """
    this_dir = os.path.join(os.path.dirname(__file__))

    styles = os.path.join(this_dir, "..", "styles", style)
    styles = os.path.normpath(styles)

    if os.path.isfile(styles):
        with open(styles) as file_:
            widget.setStyleSheet(file_.read())


def set_tooltip(widget, key):
    """Apply tooltip using given key.

    Args:
        widget (QtWidgets.QWidget): Widget to apply tooltip to.
        key (str): Key to use for tooltip lookup.

    """
    widget.setToolTip(TOOLTIPS[key])


def resize_textedit(textedit):
    """Auto grow/ shrink text edit to the current document size.

    Args:
        textedit (QtWidgets.QTextEdit): TextEdit to auto resize.

    """
    font = textedit.document().defaultFont()
    font_metrics = QtGui.QFontMetrics(font)
    text_size = font_metrics.size(0, textedit.toPlainText())
    text_height = text_size.height() + 30

    # Don't apply a resize when under the minimum value because we never want
    # to go under this value and we don't want flickering when a resize is
    # actually not needed.
    if text_height < 200:
        return

    textedit.setMinimumHeight(text_height)


def remove_highlight_border(widget):
    """Remove default highlight border for given widget when being selected.

    Args:
        widget (QtWidgets.QWidget): Widget to remove the default highlight
            border when being selected.

    """
    widget.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 0)
