"""Model to load the configuration dict.

If the environment variable 'SMART_RESCUE_CONFIG_PATH' sets a path to a
configuration file then this path will be preferred. Otherwise we use the
configuration file in this package.

Usage:
    >>> from smartRescue.mvc.model import Model

    # Let's create a configuration instance.
    >>> model = Model()

    # We can now get the configuration data using:
    >>> model.data

    # To get a list of all the steps that are enabled we can use:
    >>> model.get_enabled_stepwidgets()

"""

# Import built-in modules
import json

# Import local modules
from smartRescue import helper
from smartRescue.constants import GENERAL
from smartRescue.constants import STEPS


class Model(object):
    """Model to load the configuration dict."""

    def __init__(self):
        """Initialize the Configuration instance."""
        self.path, self.data = helper.get_config()
        self.general = self.data["general"]
        self.verbosity = self.general["logging"]
        self.ignore_prefix = self.general["ignore_file_prefix"]
        self.overwrite = self.general["overwrite"]
        self.report = self.general["create_report"]
        self.nuke_root = self.general["nuke_root_dir"]

    def config_info(self, enabled_steps):
        """Return string representation of the configuration.

        Args:
            enabled_steps (:obj:`list` of :obj:`smartRescue.steps.<Step>`):
                Step instances that are enabled and will be run.

        Returns:
            str: String representation of the configuration.

        """
        return ("Verbosity: {}\n"
                "ignore_prefix: {}\n"
                "overwrite: {}\n\n"
                "Enabled steps to use:\n\n{}"
                "".format(self.verbosity, self.ignore_prefix, self.overwrite,
                          self.info_enabled_steps(enabled_steps)))

    @staticmethod
    def get_enabled_steps(steps):
        """Get all enabled steps from given steps.

        Returns:
            list of smartRescue.steps.<stepclass>: All enabled steps
                instances.

        """
        return [step for step in steps
                if step.is_enabled]

    @staticmethod
    def info_enabled_steps(enabled_steps):
        """Get string representation of all enabled steps.

        Args:
            enabled_steps (:obj:`list` of :obj:`smartRescue.steps.<Step>`):
                Step instances that are enabled and will be run.

        Returns:
            str: String representation of all enabled steps.

        """
        return "\n".join(str(step) for step in enabled_steps)


def update_general(path, key, value):
    """Update the general configuration's key with the given value.

    Args:
        path (str): Absolute path of the configuration file to update.
        key (str): Name of the key to update.
        value (str): New value to update the given key to.

    """
    with open(path, "r") as file_:
        config_data = json.load(file_)
        config_data[GENERAL][key] = value
    with open(path, "w") as file_:
        file_.write(json.dumps(config_data, indent=4))


def update_step(path, class_name, key, value):
    """Update the step configuration for the given class_name.

    Args:
        path (str): Absolute path of the configuration file to update.
        class_name (str): Name of the class to update the configuration for.
        key (str): Name of the key to update.
        value (str): New value to update the given key to.

    """
    with open(path, "r") as file_:
        config_data = json.load(file_)
        for step in config_data[STEPS]:
            if step["class"] == class_name:
                step[key] = value
    with open(path, "w") as file_:
        file_.write(json.dumps(config_data, indent=4))
