"""Main window where you can set up and run the smartRescue."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartRescue import helper
from smartRescue.mvc import gui_helper
from smartRescue.mvc import widgets

# Icons for this package.
ICONS = helper.load_icons()

# We want to keep the class more readable and not initialize all ui members in
# init method. This is the main window which holds multiple members; There is
# no way around this.
# pylint: disable=attribute-defined-outside-init, too-many-instance-attributes
class View(QtWidgets.QWidget):
    """Main window."""

    key_pressed = QtCore.Signal(str)

    def __init__(self):
        """Initialize the View instance."""
        super(View, self).__init__()

        self.setup()
        self.build_ui()

    def setup(self):
        """Setup ui."""
        self.setMinimumSize(QtCore.QSize(1400, 800))
        self.setWindowTitle("smartRescue")
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        self.setWindowIcon(QtGui.QIcon(ICONS["logo"]))
        self.setProperty("style", "window")

    def build_ui(self):
        """Build ui."""
        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.label_startup = widgets.DropLabel()
        self.group_steps = QtWidgets.QGroupBox()

        self.group_steps_wrapper = QtWidgets.QScrollArea()
        self.group_steps_wrapper.setWidget(self.group_steps)
        self.group_steps_wrapper.setWidgetResizable(True)

        self.group_settings = QtWidgets.QGroupBox()
        self.group_settings.setProperty("style", "settings")
        self.group_settings.setVisible(False)

        self.settings = Settings()

        self.text_log = widgets.TextAreaLogger()
        self.text_log.setReadOnly(True)

        self.input_process_path = QtWidgets.QLineEdit()
        gui_helper.remove_highlight_border(self.input_process_path)
        self.push_browse_process_path = QtWidgets.QToolButton()
        gui_helper.set_icon(self.push_browse_process_path, ICONS["folder"])

        self.push_settings = QtWidgets.QToolButton()
        gui_helper.set_icon(self.push_settings, ICONS["settings"])

        self.push_process = QtWidgets.QPushButton("process")
        self.push_process.setProperty("style", "action_button")

        self.push_save_log = QtWidgets.QToolButton()
        gui_helper.set_icon(self.push_save_log, ICONS["save"])
        self.push_flush_log = QtWidgets.QToolButton()
        gui_helper.set_icon(self.push_flush_log, ICONS["erase"])

    def create_layouts(self):
        """Create layouts."""
        self.group_startup = QtWidgets.QGroupBox()
        self.group_main = QtWidgets.QGroupBox()
        self.group_main.setVisible(False)

        self.layout_master = QtWidgets.QVBoxLayout()
        self.layout_master.addWidget(self.group_startup)
        self.layout_master.addWidget(self.group_main)

        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_middle_main = QtWidgets.QHBoxLayout()
        self.layout_main_left = QtWidgets.QVBoxLayout()
        self.layout_top = QtWidgets.QHBoxLayout()
        self.layout_steps_main = QtWidgets.QVBoxLayout()
        self.layout_steps_stepwidgets = QtWidgets.QVBoxLayout()
        self.layout_settings = QtWidgets.QVBoxLayout()
        self.layout_log = QtWidgets.QVBoxLayout()

        self.layout_startup = QtWidgets.QVBoxLayout()

        self.group_main.setLayout(self.layout_main)
        self.group_startup.setLayout(self.layout_startup)

        self.layout_startup.addWidget(self.label_startup)

        self.layout_main.addLayout(self.layout_top)
        self.layout_main.addLayout(self.layout_middle_main)

        self.layout_middle_main.addLayout(self.layout_main_left)
        self.layout_middle_main.addLayout(self.layout_log)

        self.layout_main_left.addWidget(self.group_steps_wrapper)
        self.layout_main_left.addWidget(self.group_settings)
        self.layout_main_left.addWidget(self.push_process)

        self.group_steps.setLayout(self.layout_steps_main)
        self.group_settings.setLayout(self.layout_settings)

        self.layout_steps_main.addLayout(self.layout_steps_stepwidgets)
        self.layout_steps_main.addStretch()

        self.layout_settings.addWidget(self.settings)

        self.layout_top.addWidget(self.input_process_path)
        self.layout_top.addWidget(self.push_browse_process_path)
        self.layout_top.addWidget(self.push_settings)
        self.layout_top.addWidget(self.push_save_log)
        self.layout_top.addWidget(self.push_flush_log)

        self.layout_log.addWidget(self.text_log)

        self.setLayout(self.layout_master)
        gui_helper.set_style_sheet(self)

    # We need an arbitrary args argument in here as an additional argument
    # will be send when we start cythonizing this module.
    def keyPressEvent(self, event, *args):  # pylint: disable=unused-argument
        """Overwrite keyPressEvent to close window when hitting escape."""
        if event.key() == QtCore.Qt.Key_Escape:
            self.key_pressed.emit("esc")


class Settings(QtWidgets.QGroupBox):
    """Settings section widget."""

    def __init__(self):
        """Initialize the settings section."""
        super(Settings, self).__init__()

        self.build_ui()

    def build_ui(self):
        """Build ui."""
        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.label_logging = QtWidgets.QLabel("Logging")
        self.label_logging.setProperty("style", "min_150")
        self.combo_logging = QtWidgets.QComboBox()

        self.label_ignore = QtWidgets.QLabel("Ignore file prefix")
        self.label_ignore.setProperty("style", "min_150")
        self.input_ignore = QtWidgets.QLineEdit()

        self.check_overwrite = QtWidgets.QCheckBox("Overwrite")
        self.check_report = QtWidgets.QCheckBox("Create report")

        self.label_nuke_root = QtWidgets.QLabel("Nuke root dir")
        self.label_nuke_root.setProperty("style", "min_150")
        self.input_nuke_root = QtWidgets.QLineEdit()
        self.push_test_nuke_root = QtWidgets.QPushButton("test")

        self.push_web_cragl = QtWidgets.QPushButton("cragl")
        self.push_web_docs = QtWidgets.QPushButton("documentation")

        # Get rid of the selection border when being active.
        gui_helper.remove_highlight_border(self.input_ignore)
        gui_helper.remove_highlight_border(self.input_nuke_root)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_logging = QtWidgets.QHBoxLayout()
        self.layout_ignore_prefix = QtWidgets.QHBoxLayout()
        self.layout_nuke_root_dir = QtWidgets.QHBoxLayout()
        self.layout_web = QtWidgets.QHBoxLayout()

        self.layout_main.addLayout(self.layout_logging)
        self.layout_main.addLayout(self.layout_ignore_prefix)
        self.layout_main.addWidget(self.check_overwrite)
        self.layout_main.addWidget(self.check_report)
        self.layout_main.addLayout(self.layout_nuke_root_dir)
        self.layout_main.addStretch()
        self.layout_main.addLayout(self.layout_web)

        self.layout_logging.addWidget(self.label_logging)
        self.layout_logging.addWidget(self.combo_logging)

        self.layout_ignore_prefix.addWidget(self.label_ignore)
        self.layout_ignore_prefix.addWidget(self.input_ignore)

        self.layout_nuke_root_dir.addWidget(self.label_nuke_root)
        self.layout_nuke_root_dir.addWidget(self.input_nuke_root)
        self.layout_nuke_root_dir.addWidget(self.push_test_nuke_root)

        self.layout_web.addWidget(self.push_web_cragl)
        self.layout_web.addWidget(self.push_web_docs)

        self.setLayout(self.layout_main)
        gui_helper.set_style_sheet(self)
