"""Custom widgets."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartRescue import helper
from smartRescue.mvc import gui_helper

# Icons for this package.
ICONS = helper.load_icons()


# We want to keep the class more readable and not initialize all ui members in
# the init method. This is a main widget which holds multiple members; There
# is no way around this.
# pylint: disable=attribute-defined-outside-init, too-many-instance-attributes
class StepWidget(QtWidgets.QWidget):
    """Widget representing a step.

    This widget contains a checkbox to enable/ disable the step, the label
    showing the name of the step, a tooltip showing the description of the
    step and a text area to modify the step's setup.

    """

    def __init__(self, parent_controller):
        """Initialize the StepWidget instance.

        Args:
            parent_controller (smartRescue.mvc.controller.Controller): The
                parent controller of the main view so that we can send
                responses back.

        """
        super(StepWidget, self).__init__()

        self.parent_controller = parent_controller
        self.build_ui()

    def build_ui(self):
        """Build ui."""
        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.check = QtWidgets.QCheckBox()
        self.push_toggle_body = QtWidgets.QToolButton()
        self.push_toggle_body.setIcon(QtGui.QIcon(ICONS["arrow_down"]))

        self.group_body = QtWidgets.QGroupBox()
        self.group_body.setVisible(False)

        self.label_docs_header = QtWidgets.QLabel()
        self.label_docs_header.setWordWrap(True)
        self.label_docs_header.setTextInteractionFlags(
            QtCore.Qt.TextSelectableByMouse)
        self.label_docs_header.setProperty("style", "description")

        self.label_docs_body = QtWidgets.QLabel()
        self.label_docs_body.setWordWrap(True)
        self.label_docs_body.setTextInteractionFlags(
            QtCore.Qt.TextSelectableByMouse)
        self.label_docs_body.setProperty("style", "description")
        self.label_docs_body.setVisible(False)

        self.label_docs_example_standard = QtWidgets.QLabel()
        self.label_docs_example_standard.setWordWrap(True)
        self.label_docs_example_standard.setTextInteractionFlags(
            QtCore.Qt.TextSelectableByMouse)
        self.label_docs_example_standard.setProperty("style", "description")
        self.label_docs_example_standard.setVisible(False)

        self.label_docs_example_advanced = QtWidgets.QLabel()
        self.label_docs_example_advanced.setWordWrap(True)
        self.label_docs_example_advanced.setTextInteractionFlags(
            QtCore.Qt.TextSelectableByMouse)
        self.label_docs_example_advanced.setProperty("style", "description")
        self.label_docs_example_advanced.setVisible(False)

        self.push_toggle_mode = QtWidgets.QToolButton()
        self.push_info = QtWidgets.QToolButton()
        gui_helper.set_icon(self.push_info, ICONS["info"])
        self.push_test = QtWidgets.QToolButton()
        gui_helper.set_icon(self.push_test, ICONS["test_grey"])

        self.text_setup = QtWidgets.QTextEdit()
        self.text_setup.setProperty("style", "text_setup")
        self.text_setup.setMinimumHeight(200)
        self.text_setup.setTabStopWidth(30)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_top = QtWidgets.QHBoxLayout()
        self.layout_docs = QtWidgets.QVBoxLayout()
        self.layout_docs_container = QtWidgets.QVBoxLayout()
        self.layout_buttons = QtWidgets.QHBoxLayout()
        self.layout_buttons_container = QtWidgets.QVBoxLayout()
        self.layout_body = QtWidgets.QVBoxLayout()
        self.group_setup_widgets = QtWidgets.QGroupBox()
        self.layout_setup_widgets = QtWidgets.QVBoxLayout()
        self.group_setup_widgets.setLayout(self.layout_setup_widgets)

        self.layout_main.addLayout(self.layout_top)
        self.layout_main.addWidget(self.group_body)

        self.layout_top.addWidget(self.check)
        self.layout_top.addWidget(self.push_toggle_body)

        self.layout_docs_examples = QtWidgets.QVBoxLayout()
        self.layout_docs_examples.addWidget(self.label_docs_example_standard)
        self.layout_docs_examples.addWidget(self.label_docs_example_advanced)
        self.group_docs_examples = QtWidgets.QGroupBox()
        self.group_docs_examples.setVisible(False)
        self.group_docs_examples.setLayout(self.layout_docs_examples)

        self.layout_docs.addWidget(self.label_docs_header)
        self.layout_docs.addWidget(self.label_docs_body)
        self.layout_docs.addWidget(self.group_docs_examples)

        self.layout_docs_container.addLayout(self.layout_docs, 99)
        self.layout_docs.addLayout(self.layout_buttons_container)

        self.layout_buttons.addStretch()
        self.layout_buttons.addWidget(self.push_info)
        self.layout_buttons.addWidget(self.push_test)
        self.layout_buttons.addWidget(self.push_toggle_mode)

        self.layout_buttons_container.addLayout(self.layout_buttons)
        self.layout_buttons_container.addStretch()

        self.layout_body.addLayout(self.layout_docs_container)
        self.layout_body.addWidget(self.text_setup)
        self.layout_body.addWidget(self.group_setup_widgets)

        self.group_body.setLayout(self.layout_body)

        self.setLayout(self.layout_main)


class TextAreaLogger(QtWidgets.QTextEdit):
    """Text area that can receive logging information."""

    def __init__(self):
        """Initialize the TextAreaLogger instance."""
        super(TextAreaLogger, self).__init__()
        self.widget = QtWidgets.QTextEdit()

    def emit(self, content):
        """Append the given content to the widget.

        Args:
            content (str): The content to append to the widget.

        """
        text = self.format(content)
        self.widget.setText(self.widget.toPlainText() + (text))


class DropLabel(QtWidgets.QLabel):
    """Custom label that supports dropping and adds a file browser."""

    file_dropped = QtCore.Signal(str)
    clicked = QtCore.Signal()

    def __init__(self):
        """Initialize the DropLabel instance."""
        super(DropLabel, self).__init__()

        self.setAcceptDrops(True)

        self.setAlignment(QtCore.Qt.AlignCenter)
        self.set_style(hover=False)

        icon = ICONS["logo_dark"]
        self.setPixmap(QtGui.QPixmap.fromImage(QtGui.QImage(icon)))
        self.setToolTip("Drag folder / file or click to browse")

    def set_style(self, hover=True):
        """Set style for the widget.

        Args:
            mode (bool, optional): If True use hover style, otherwise use
                normal style.

        """
        style = "startup_hover" if hover else "startup"
        self.setProperty("style", style)
        gui_helper.set_style_sheet(self)

    def dragEnterEvent(self, event):
        """Overwrite dragEnterEvent to set mimeData."""
        event.accept()
        self.set_style(hover=True)

    def dragLeaveEvent(self, event):
        """Overwrite dragEnterEvent to set mimeData."""
        event.accept()
        self.set_style(hover=False)

    def dragMoveEvent(self, event):
        """Overwrite dragMoveEvent to accept drags."""
        event.setDropAction(QtCore.Qt.CopyAction)
        event.accept()

    def dropEvent(self, event):
        """Fire file_dropped signal."""
        event.setDropAction(QtCore.Qt.CopyAction)
        event.accept()

        self.set_style(hover=False)

        links = []
        for url in event.mimeData().urls():
            links.append(str(url.toLocalFile()))

        self.file_dropped.emit(links[0])

    def mousePressEvent(self, event):
        """Emit signal when clicked the label.

        Args:
            event (QtQore.QEvent): Event instance.

        """
        super(DropLabel, self).mousePressEvent(event)
        self.clicked.emit()
