"""Processor module to process the jobs.

This module gets called by Nuke's terminal mode.

"""

# Import built-in modules

import logging
import json
import os
import sys

# Import smartRescue root dir because we require it and calling it from a
# subprocess has smartRescue not available in the PYTHONPATH.
# pylint: disable=wrong-import-position
_THIS_DIR = os.path.dirname(__file__)
_SMARTRESCUE = os.path.normpath(os.path.join(_THIS_DIR, ".."))
sys.path.append(_SMARTRESCUE)

# Import local modules
from smartRescue.constants import LOGGING_LEVEL
# This import is used in get_class() to get the class instance by the specified
# name using getattr.
import smartRescue.steps  # pylint:disable=unused-import
from smartRescue import helper

LOGGER = logging.getLogger("process")


def main(job_file):
    """Entry point to start the process.

    This will read the job json file and trigger the process.

    Args:
        job_file (str): Absolute path of process file that contains the
            steps class names and the setup to apply to these.

    """
    with open(job_file, "r") as file_:
        job = json.load(file_)

    LOGGER.setLevel(LOGGING_LEVEL[job["logging"]])
    LOGGER.info("Starting smartRescue")

    if job["create_report"]:
        LOGGER.info("Writing output log to %s", job["logging_path"])

    LOGGER.info("Using process folder: %s", job["process_path"])
    LOGGER.info("Applying %s steps", len(job["steps"]))
    LOGGER.debug("Job configuration: %s", json.dumps(job, indent=4))

    process(job)


def process(job):
    """Process the job.

    Args:
        job (dict): Job dictionary containing all required information in the
            format:
                {
                    'process_path': '/Users/cragl/Desktop/process',
                    'logging':
                    'debug',
                    'ignore_file_prefix': '_',
                    'logging_path': '/tempath/cragl_smartrescue/id/job.log',
                    'create_report': True,
                    'overwrite': False,
                    'steps': [
                        {
                            '_class_name': 'AllNodes',
                            'mode': 'disable',
                            'keep': []
                        }
                    ],
                }

    """
    nuke_scripts = _get_nuke_scripts(job)
    LOGGER.info("Process working files: %s", nuke_scripts)

    for path in nuke_scripts:
        LOGGER.info("Process working file: %s", path)
        for setup in job["steps"]:
            class_ = get_class(setup)
            step = class_(path, setup, job)
            step.run(perform_file_handle=True)


def get_class(setup):
    """Get the step class by the _class_name being nested in the setup.

    Args:
        setup (dict): The setup of the job to process.

    Returns:
        smartRescue.steps.<Step>: Step class being resolved from look up of
            the given _class_name.

    """
    class_name = setup.get("_class_name")

    try:
        class_ = getattr(sys.modules["smartRescue.steps"], class_name)
    except AttributeError:
        raise ImportError(
            "No such step class '{}' defined in smartRescue.steps.__init__."
            "Please make sure that this step is added to the list."
            "".format(class_name))

    return class_


def _get_nuke_scripts(job):
    """Get the Nuke scripts to process.

    If overwrite is on, process directly on the .nk files, otherwise create a
    backup file and process on this one.

    """
    nuke_scripts = helper.get_nuke_scripts(job["process_path"],
                                           job["ignore_file_prefix"])

    if job["overwrite"]:
        return nuke_scripts

    return helper.create_working_file_copies(nuke_scripts)


if __name__ == "__main__":
    JOB_FILE = sys.argv[1]
    main(JOB_FILE)
